@@ -1,7 +1,7 @@
 /*
  * dfu-programmer
  *
- * $Id: arguments.c 55 2007-08-13 04:02:33Z schmidtw $
+ * $Id: arguments.c 79 2008-12-11 05:55:12Z schmidtw $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,64 +18,101 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include <stdbool.h>
+#include <stddef.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "dfu-bool.h"
+#include "dfu-device.h"
 #include "config.h"
 #include "arguments.h"
 
 struct option_mapping_structure {
     const char *name;
-    int value;
+    int32_t value;
 };
 
 struct target_mapping_structure {
     const char *name;
     enum targets_enum value;
-    enum device_type_enum device_type;
-    unsigned short chip_id;
-    unsigned short vendor_id;
-    unsigned int memory_size;
-    unsigned short flash_page_size;
-    bool initial_abort;
-    bool honor_interfaceclass;
+    atmel_device_class_t device_type;
+    uint16_t chip_id;
+    uint16_t vendor_id;
+    size_t memory_size;
+    size_t bootloader_size;
+    dfu_bool bootloader_at_highmem;
+    size_t flash_page_size;
+    dfu_bool initial_abort;
+    dfu_bool honor_interfaceclass;
+    size_t eeprom_page_size;
+    size_t eeprom_memory_size;
 };
 
 /* NOTE FOR: at90usb1287, at90usb1286, at90usb647, at90usb646, at90usb162, at90usb82
  *
  * The actual size of the user-programmable section is limited by the
  * space needed by the bootloader.  The size of the bootloader is set
- * by BOOTSZ0/BOOTSZ1 fuse bits; here we assume the factory default and
- * maximum of 4K words.  The window used for the bootloader is at the top
- * of the of memory.
+ * by BOOTSZ0/BOOTSZ1 fuse bits; here we assume the bootloader is 4kb or 8kb.
+ * The window used for the bootloader is at the top of the of memory.
  */
 
 /* ----- target specific structures ----------------------------------------- */
 static struct target_mapping_structure target_map[] = {
-    { "at89c51snd1c", tar_at89c51snd1c, device_8051, 0x2FFF, 0x03eb, 0x10000, 128, false, true },
-    { "at89c5130",    tar_at89c5130,    device_8051, 0x2FFD, 0x03eb, 0x04000, 128, false, true },
-    { "at89c5131",    tar_at89c5131,    device_8051, 0x2FFD, 0x03eb, 0x08000, 128, false, true },
-    { "at89c5132",    tar_at89c5132,    device_8051, 0x2FFF, 0x03eb, 0x10000, 128, false, true },
-    { "at90usb1287",  tar_at90usb1287,  device_AVR,  0x2FFB, 0x03eb, 0x1E000, 128, true, false },
-    { "at90usb1286",  tar_at90usb1286,  device_AVR,  0x2FFB, 0x03eb, 0x1E000, 128, true, false },
-    { "at90usb647",   tar_at90usb647,   device_AVR,  0x2FFB, 0x03eb, 0x0E000, 128, true, false },
-    { "at90usb646",   tar_at90usb646,   device_AVR,  0x2FFB, 0x03eb, 0x0E000, 128, true, false },
-    { "at90usb162",   tar_at90usb162,   device_AVR,  0x2FFA, 0x03eb, 0x03000, 128, true, false },
-    { "at90usb82",    tar_at90usb82,    device_AVR,  0x2FFA, 0x03eb, 0x01000, 128, true, false },
+    { "at89c51snd1c",   tar_at89c51snd1c,   adc_8051,  0x2FFF, 0x03eb, 0x10000, 0x1000, true,  128, false, true,  0,   0      },
+    { "at89c51snd2c",   tar_at89c51snd2c,   adc_8051,  0x2FFF, 0x03eb, 0x10000, 0x1000, true,  128, false, true,  0,   0      },
+    { "at89c5130",      tar_at89c5130,      adc_8051,  0x2FFD, 0x03eb, 0x04000, 0x0000, true,  128, false, true,  128, 0x03FF },    /* The bootloader is out of the normal flash. */
+    { "at89c5131",      tar_at89c5131,      adc_8051,  0x2FFD, 0x03eb, 0x08000, 0x0000, true,  128, false, true,  128, 0x03FF },    /* The bootloader is out of the normal flash. */
+    { "at89c5132",      tar_at89c5132,      adc_8051,  0x2FFF, 0x03eb, 0x10000, 0x0C00, true,  128, false, true,  0,   0      },
+    { "at90usb1287",    tar_at90usb1287,    adc_AVR,   0x2FFB, 0x03eb, 0x20000, 0x2000, true,  128, true,  false, 128, 0x0FFF },
+    { "at90usb1286",    tar_at90usb1286,    adc_AVR,   0x2FFB, 0x03eb, 0x20000, 0x2000, true,  128, true,  false, 128, 0x0FFF },
+    { "at90usb1287-4k", tar_at90usb1287_4k, adc_AVR,   0x2FFB, 0x03eb, 0x20000, 0x1000, true,  128, true,  false, 128, 0x0FFF },
+    { "at90usb1286-4k", tar_at90usb1286_4k, adc_AVR,   0x2FFB, 0x03eb, 0x20000, 0x1000, true,  128, true,  false, 128, 0x0FFF },
+    { "at90usb647",     tar_at90usb647,     adc_AVR,   0x2FF9, 0x03eb, 0x10000, 0x2000, true,  128, true,  false, 128, 0x07FF },
+    { "at90usb646",     tar_at90usb646,     adc_AVR,   0x2FF9, 0x03eb, 0x10000, 0x2000, true,  128, true,  false, 128, 0x07FF },
+    { "at90usb162",     tar_at90usb162,     adc_AVR,   0x2FFA, 0x03eb, 0x04000, 0x1000, true,  128, true,  false, 128, 0x01FF },
+    { "at90usb82",      tar_at90usb82,      adc_AVR,   0x2FF7, 0x03eb, 0x02000, 0x1000, true,  128, true,  false, 128, 0x01FF },
+    { "atmega32u6",     tar_atmega32u6,     adc_AVR,   0x2FF2, 0x03eb, 0x08000, 0x1000, true,  128, true,  false, 128, 0x03FF },
+    { "atmega32u4",     tar_atmega32u4,     adc_AVR,   0x2FF4, 0x03eb, 0x08000, 0x1000, true,  128, true,  false, 128, 0x03FF },
+    { "atmega16u4",     tar_atmega16u4,     adc_AVR,   0x2FF3, 0x03eb, 0x04000, 0x1000, true,  128, true,  false, 128, 0x01FF },
+    { "at32uc3a0128",   tar_at32uc3a0128,   adc_AVR32, 0x2FF8, 0x03eb, 0x20000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a1128",   tar_at32uc3a1128,   adc_AVR32, 0x2FF8, 0x03eb, 0x20000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a0256",   tar_at32uc3a0256,   adc_AVR32, 0x2FF8, 0x03eb, 0x40000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a1256",   tar_at32uc3a1256,   adc_AVR32, 0x2FF8, 0x03eb, 0x40000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a0512",   tar_at32uc3a0512,   adc_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a1512",   tar_at32uc3a1512,   adc_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a0512es", tar_at32uc3a0512es, adc_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a1512es", tar_at32uc3a1512es, adc_AVR32, 0x2FF8, 0x03eb, 0x80000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a364",    tar_at32uc3a364,    adc_AVR32, 0x2FF1, 0x03eb, 0x10000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a364s",   tar_at32uc3a364s,   adc_AVR32, 0x2FF1, 0x03eb, 0x10000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a3128",   tar_at32uc3a3128,   adc_AVR32, 0x2FF1, 0x03eb, 0x20000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a3128s",  tar_at32uc3a3128s,  adc_AVR32, 0x2FF1, 0x03eb, 0x20000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a3256",   tar_at32uc3a3256,   adc_AVR32, 0x2FF1, 0x03eb, 0x40000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3a3256s",  tar_at32uc3a3256s,  adc_AVR32, 0x2FF1, 0x03eb, 0x40000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3b064",    tar_at32uc3b064,    adc_AVR32, 0x2FF6, 0x03eb, 0x10000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3b164",    tar_at32uc3b164,    adc_AVR32, 0x2FF6, 0x03eb, 0x10000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3b0128",   tar_at32uc3b0128,   adc_AVR32, 0x2FF6, 0x03eb, 0x20000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3b1128",   tar_at32uc3b1128,   adc_AVR32, 0x2FF6, 0x03eb, 0x20000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3b0256",   tar_at32uc3b0256,   adc_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3b1256",   tar_at32uc3b1256,   adc_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3b0256es", tar_at32uc3b0256es, adc_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 128, false, true,  0,   0      },
+    { "at32uc3b1256es", tar_at32uc3b1256es, adc_AVR32, 0x2FF6, 0x03eb, 0x40000, 0x2000, false, 128, false, true,  0,   0      },
     { NULL }
 };
 
 /* ----- command specific structures ---------------------------------------- */
 static struct option_mapping_structure command_map[] = {
-    { "configure", com_configure },
-    { "dump",      com_dump      },
-    { "erase",     com_erase     },
-    { "flash",     com_flash     },
-    { "get",       com_get       },
-    { "start",     com_start_app },
-    { "version",   com_version   },
+    { "configure",    com_configure },
+    { "dump",         com_dump      },
+    { "dump-eeprom",  com_edump     },
+    { "erase",        com_erase     },
+    { "flash",        com_flash     },
+    { "flash-eeprom", com_eflash    },
+    { "get",          com_get       },
+    { "reset",        com_reset     },
+    { "start",        com_start_app },
+    { "version",      com_version   },
     { NULL }
 };
 
@@ -117,7 +154,7 @@ static void usage()
     fprintf( stderr, "Usage: dfu-programmer target command [command-options] "
                      "[global-options] [file|data]\n" );
     fprintf( stderr, "targets:\n" );
-    while( 0 != *((int *) map) ) {
+    while( 0 != *((int32_t *) map) ) {
         fprintf( stderr, "        %s\n", map->name );
         map++;
     }
@@ -127,23 +164,28 @@ static void usage()
                      "[--suppress-validation] [global-options] data\n" );
     fprintf( stderr, "        dump "
                      "[global-options]\n" );
+    fprintf( stderr, "        dump-eeprom "
+                     "[global-options]\n" );
     fprintf( stderr, "        erase "
-                     "[--suppress-validation] [global-options]\n" );
+                     "[--suppress-validation] [--suppress-bootloader-mem] [global-options]\n" );
     fprintf( stderr, "        flash "
                      "[--suppress-validation] [global-options] file\n" );
+    fprintf( stderr, "        flash-eeprom "
+                     "[--suppress-validation] [global-options] file\n" );
     fprintf( stderr, "        get {bootloader-version|ID1|ID2|BSB|SBV|SSB|EB|\n"
                      "            manufacturer|family|product-name|\n"
                      "            product-revision|HSB} "
                      "[global-options]\n" );
+    fprintf( stderr, "        reset [global-options]\n" );
     fprintf( stderr, "        start [global-options]\n" );
     fprintf( stderr, "        version [global-options]\n" );
 }
 
-static int assign_option( int *arg,
-                          char *value,
-                          struct option_mapping_structure *map )
+static int32_t assign_option( int32_t *arg,
+                              char *value,
+                              struct option_mapping_structure *map )
 {
-    while( 0 != *((int *) map) ) {
+    while( 0 != *((int32_t *) map) ) {
         if( 0 == strcasecmp(value, map->name) ) {
             *arg = map->value;
             return 0;
@@ -156,30 +198,50 @@ static int assign_option( int *arg,
 }
 
 
-static int assign_target( struct programmer_arguments *args,
-                          char *value,
-                          struct target_mapping_structure *map )
+static int32_t assign_target( struct programmer_arguments *args,
+                              char *value,
+                              struct target_mapping_structure *map )
 {
-    while( 0 != *((int *) map) ) {
+    while( 0 != *((int32_t *) map) ) {
         if( 0 == strcasecmp(value, map->name) ) {
             args->target  = map->value;
             args->chip_id = map->chip_id;
             args->vendor_id = map->vendor_id;
             args->device_type = map->device_type;
-            args->memory_size = map->memory_size;
+            args->eeprom_memory_size = map->eeprom_memory_size;
             args->flash_page_size = map->flash_page_size;
+            args->eeprom_page_size = map->eeprom_page_size;
             args->initial_abort = map->initial_abort;
             args->honor_interfaceclass = map->honor_interfaceclass;
-            args->top_memory_address = map->memory_size - 1;
+            args->flash_address_top = map->memory_size;
+            args->flash_address_bottom = 0;
+            args->bootloader_bottom = 0;
+            args->bootloader_top = 0;
+            if( true == map->bootloader_at_highmem ) {
+                args->bootloader_bottom = args->flash_address_top - map->bootloader_size;
+                args->bootloader_top = args->flash_address_top;
+            } else {
+                args->bootloader_bottom = args->flash_address_bottom;
+                args->bootloader_top += map->bootloader_size;
+            }
+            if( 0 < map->eeprom_memory_size ) {
+                args->top_eeprom_memory_address = map->eeprom_memory_size - 1;
+            } else {
+                args->top_eeprom_memory_address = 0;
+            }
             switch( args->device_type ) {
-                case device_8051:
+                case adc_8051:
                     strncpy( args->device_type_string, "8051",
                              DEVICE_TYPE_STRING_MAX_LENGTH );
                     break;
-                case device_AVR:
+                case adc_AVR:
                     strncpy( args->device_type_string, "AVR",
                              DEVICE_TYPE_STRING_MAX_LENGTH );
                     break;
+                case adc_AVR32:
+                    strncpy( args->device_type_string, "AVR32",
+                             DEVICE_TYPE_STRING_MAX_LENGTH );
+                    break;
             }
             return 0;
         }
@@ -191,11 +253,11 @@ static int assign_target( struct programmer_arguments *args,
 }
 
 
-static int assign_global_options( struct programmer_arguments *args,
-                                  int argc,
-                                  char **argv )
+static int32_t assign_global_options( struct programmer_arguments *args,
+                                      const size_t argc,
+                                      char **argv )
 {
-    int i = 0;
+    size_t i = 0;
 
     /* Find '--quiet' if it is here */
     for( i = 0; i < argc; i++ ) {
@@ -205,6 +267,16 @@ static int assign_global_options( struct programmer_arguments *args,
             break;
         }
     }
+	
+	/* Find '--suppress-bootloader-mem' if it is here */
+    for( i = 0; i < argc; i++ ) {
+        if( 0 == strcmp("--suppress-bootloader-mem", argv[i]) ) {
+            *argv[i] = '\0';
+            args->suppressbootloader = 1;
+            break;
+        }
+    }
+	
     /* Find '--suppress-validation' if it is here - even though it is not
      * used by all this is easier. */
     for( i = 0; i < argc; i++ ) {
@@ -219,6 +291,7 @@ static int assign_global_options( struct programmer_arguments *args,
                     args->com_erase_data.suppress_validation = 1;
                     break;
                 case com_flash:
+                case com_eflash:
                     args->com_flash_data.suppress_validation = 1;
                     break;
                 default:
@@ -256,20 +329,20 @@ static int assign_global_options( struct programmer_arguments *args,
 }
 
 
-static int assign_com_configure_option( struct programmer_arguments *args,
-                                        int parameter,
-                                        char *value )
+static int32_t assign_com_configure_option( struct programmer_arguments *args,
+                                            const int32_t parameter,
+                                            char *value )
 {
     /* name & value */
     if( 0 == parameter ) {
         /* name */
-        if( 0 != assign_option((int *) &(args->com_configure_data.name),
+        if( 0 != assign_option((int32_t *) &(args->com_configure_data.name),
                                value, configure_map) )
         {
             return -1;
         }
     } else {
-        int temp = 0;
+        int32_t temp = 0;
         /* value */
         if( 1 != sscanf(value, "%i", &(temp)) )
             return -2;
@@ -285,9 +358,9 @@ static int assign_com_configure_option( struct programmer_arguments *args,
 }
 
 
-static int assign_com_flash_option( struct programmer_arguments *args,
-                                    int parameter,
-                                    char *value )
+static int32_t assign_com_flash_option( struct programmer_arguments *args,
+                                        const int32_t parameter,
+                                        char *value )
 {
     /* file */
     args->com_flash_data.original_first_char = *value;
@@ -297,12 +370,12 @@ static int assign_com_flash_option( struct programmer_arguments *args,
 }
 
 
-static int assign_com_get_option( struct programmer_arguments *args,
-                                  int parameter,
-                                  char *value )
+static int32_t assign_com_get_option( struct programmer_arguments *args,
+                                      const int32_t parameter,
+                                      char *value )
 {
     /* name */
-    if( 0 != assign_option((int *) &(args->com_get_data.name),
+    if( 0 != assign_option((int32_t *) &(args->com_get_data.name),
                            value, get_map) )
     {
         return -1;
@@ -312,13 +385,13 @@ static int assign_com_get_option( struct programmer_arguments *args,
 }
 
 
-static int assign_command_options( struct programmer_arguments *args,
-                                   int argc,
-                                   char **argv )
+static int32_t assign_command_options( struct programmer_arguments *args,
+                                       const size_t argc,
+                                       char **argv )
 {
-    int i = 0;
-    int param = 0;
-    int required_params = 0;
+    size_t i = 0;
+    int32_t param = 0;
+    int32_t required_params = 0;
 
     /* Deal with all remaining command-specific arguments. */
     for( i = 0; i < argc; i++ ) {
@@ -333,6 +406,7 @@ static int assign_command_options( struct programmer_arguments *args,
                 break;
 
             case com_flash:
+            case com_eflash:
                 required_params = 1;
                 if( 0 != assign_com_flash_option(args, param, argv[i]) )
                     return -3;
@@ -363,7 +437,7 @@ static void print_args( struct programmer_arguments *args )
 {
     const char *command = "(unknown)";
     const char *target = "(unknown)";
-    int i;
+    size_t i;
 
     for( i = 0; i < sizeof(target_map) / sizeof(target_map[0]); i++ ) {
         if( args->target == target_map[i].value ) {
@@ -402,6 +476,7 @@ static void print_args( struct programmer_arguments *args )
                         "false" : "true" );
             break;
         case com_flash:
+        case com_eflash:
             printf( "   validate: %s\n",
                     (args->com_flash_data.suppress_validation) ?
                         "false" : "true" );
@@ -418,12 +493,12 @@ static void print_args( struct programmer_arguments *args )
 }
 
 
-int parse_arguments( struct programmer_arguments *args,
-                     int argc,
-                     char **argv )
+int32_t parse_arguments( struct programmer_arguments *args,
+                         const size_t argc,
+                         char **argv )
 {
-    int i;
-    int status = 0;
+    int32_t i;
+    int32_t status = 0;
 
     if( NULL == args )
         return -1;
@@ -432,6 +507,7 @@ int parse_arguments( struct programmer_arguments *args,
     args->target  = tar_none;
     args->command = com_none;
     args->quiet   = 0;
+    args->suppressbootloader = 0;
 
     /* Make sure there are the minimum arguments */
     if( argc < 3 ) {
@@ -444,7 +520,7 @@ int parse_arguments( struct programmer_arguments *args,
         goto done;
     }
 
-    if( 0 != assign_option((int *) &(args->command), argv[2], command_map) ) {
+    if( 0 != assign_option((int32_t *) &(args->command), argv[2], command_map) ) {
         status = -4;
         goto done;
     }
@@ -474,7 +550,7 @@ int parse_arguments( struct programmer_arguments *args,
     }
 
     /* if this is a flash command, restore the filename */
-    if( com_flash == args->command ) {
+    if( (com_flash == args->command) || (com_eflash == args->command) ) {
         if( 0 == args->com_flash_data.file ) {
             fprintf( stderr, "flash filename is missing\n" );
             status = -8;
