@@ -1,7 +1,7 @@
 /*
  * dfu-programmer
  *
- * $Id: dfu.c 48 2007-03-28 07:35:45Z schmidtw $
+ * $Id: dfu.c 75 2008-10-31 08:19:42Z schmidtw $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +20,13 @@
 
 #include <stdio.h>
 #include <stdarg.h>
-#include <stdbool.h>
+#include <stdint.h>
+#include <stddef.h>
 #include <usb.h>
 #include <errno.h>
 #include "dfu.h"
 #include "util.h"
+#include "dfu-bool.h"
 
 /* DFU commands */
 #define DFU_DETACH      0
@@ -45,41 +47,49 @@
  * before the giving up going into dfu mode. */
 #define DFU_DETACH_TIMEOUT 1000
 
-#define DFU_DEBUG_THRESHOLD 100
+#define DFU_DEBUG_THRESHOLD         100
+#define DFU_TRACE_THRESHOLD         200
+#define DFU_MESSAGE_DEBUG_THRESHOLD 300
 
 #define DEBUG(...)  dfu_debug( __FILE__, __FUNCTION__, __LINE__, \
                                DFU_DEBUG_THRESHOLD, __VA_ARGS__ )
+#define TRACE(...)  dfu_debug( __FILE__, __FUNCTION__, __LINE__, \
+                               DFU_TRACE_THRESHOLD, __VA_ARGS__ )
+#define MSG_DEBUG(...)  dfu_debug( __FILE__, __FUNCTION__, __LINE__, \
+                               DFU_MESSAGE_DEBUG_THRESHOLD, __VA_ARGS__ )
 
-static unsigned short transaction = 0;
+static uint16_t transaction = 0;
 
-static int dfu_find_interface( const struct usb_device *device,
-                               const bool honor_interfaceclass );
-static int dfu_make_idle( struct usb_dev_handle *handle,
-                          unsigned short interface,
-                          bool initial_abort );
-static void dfu_msg_response_output( const char *function, const int result );
+static int32_t dfu_find_interface( const struct usb_device *device,
+                                   const dfu_bool honor_interfaceclass );
+static int32_t dfu_make_idle( dfu_device_t *device, const dfu_bool initial_abort );
+static void dfu_msg_response_output( const char *function, const int32_t result );
 
 /*
  *  DFU_DETACH Request (DFU Spec 1.0, Section 5.1)
  *
- *  handle    - the usb_dev_handle to communicate with
- *  interface - the interface to communicate with
+ *  device    - the dfu device to commmunicate with
  *  timeout   - the timeout in ms the USB device should wait for a pending
  *              USB reset before giving up and terminating the operation
  *
  *  returns 0 or < 0 on error
  */
-int dfu_detach( struct usb_dev_handle *handle,
-                const unsigned short interface,
-                const unsigned short timeout )
+int32_t dfu_detach( dfu_device_t *device, const int32_t timeout )
 {
-    int result;
+    int32_t result;
 
-    result = usb_control_msg( handle,
+    TRACE( "%s( %p, %d )\n", __FUNCTION__, device, timeout );
+
+    if( (NULL == device) || (NULL == device->handle) || (timeout < 0) ) {
+        DEBUG( "Invalid parameter\n" );
+        return -1;
+    }
+
+    result = usb_control_msg( device->handle,
           /* bmRequestType */ USB_ENDPOINT_OUT | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
           /* bRequest      */ DFU_DETACH,
           /* wValue        */ timeout,
-          /* wIndex        */ interface,
+          /* wIndex        */ device->interface,
           /* Data          */ NULL,
           /* wLength       */ 0,
                               DFU_TIMEOUT );
@@ -93,38 +103,49 @@ int dfu_detach( struct usb_dev_handle *handle,
 /*
  *  DFU_DNLOAD Request (DFU Spec 1.0, Section 6.1.1)
  *
- *  handle    - the usb_dev_handle to communicate with
- *  interface - the interface to communicate with
+ *  device    - the dfu device to commmunicate with
  *  length    - the total number of bytes to transfer to the USB
  *              device - must be less than wTransferSize
  *  data      - the data to transfer
  *
  *  returns the number of bytes written or < 0 on error
  */
-int dfu_download( struct usb_dev_handle *handle,
-                  const unsigned short interface,
-                  const unsigned short length,
-                  char* data )
+int32_t dfu_download( dfu_device_t *device, const size_t length, uint8_t* data )
 {
-    int result;
+    int32_t result;
+
+    TRACE( "%s( %p, %u, %p )\n", __FUNCTION__, device, length, data );
 
     /* Sanity checks */
+    if( (NULL == device) || (NULL == device->handle) ) {
+        DEBUG( "Invalid parameter\n" );
+        return -1;
+    }
+
     if( (0 != length) && (NULL == data) ) {
         DEBUG( "data was NULL, but length != 0\n" );
-        return -1;
+        return -2;
     }
 
     if( (0 == length) && (NULL != data) ) {
         DEBUG( "data was not NULL, but length == 0\n" );
-        return -2;
+        return -3;
     }
 
-    result = usb_control_msg( handle,
+
+    {
+        size_t i;
+        for( i = 0; i < length; i++ ) {
+            MSG_DEBUG( "Message: m[%u] = 0x%02x\n", i, data[i] );
+        }
+    }
+
+    result = usb_control_msg( device->handle,
           /* bmRequestType */ USB_ENDPOINT_OUT | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
           /* bRequest      */ DFU_DNLOAD,
           /* wValue        */ transaction++,
-          /* wIndex        */ interface,
-          /* Data          */ data,
+          /* wIndex        */ device->interface,
+          /* Data          */ (char *) data,
           /* wLength       */ length,
                               DFU_TIMEOUT );
 
@@ -137,33 +158,36 @@ int dfu_download( struct usb_dev_handle *handle,
 /*
  *  DFU_UPLOAD Request (DFU Spec 1.0, Section 6.2)
  *
- *  handle    - the usb_dev_handle to communicate with
- *  interface - the interface to communicate with
+ *  device    - the dfu device to commmunicate with
  *  length    - the maximum number of bytes to receive from the USB
  *              device - must be less than wTransferSize
  *  data      - the buffer to put the received data in
  *
  *  returns the number of bytes received or < 0 on error
  */
-int dfu_upload( struct usb_dev_handle *handle,
-                const unsigned short interface,
-                const unsigned short length,
-                char* data )
+int32_t dfu_upload( dfu_device_t *device, const size_t length, uint8_t* data )
 {
-    int result;
+    int32_t result;
+
+    TRACE( "%s( %p, %u, %p )\n", __FUNCTION__, device, length, data );
 
     /* Sanity checks */
-    if( (0 == length) || (NULL == data) ) {
-        DEBUG( "data was NULL, or length is 0\n" );
+    if( (NULL == device) || (NULL == device->handle) ) {
+        DEBUG( "Invalid parameter\n" );
         return -1;
     }
 
-    result = usb_control_msg( handle,
+    if( (0 == length) || (NULL == data) ) {
+        DEBUG( "data was NULL, or length is 0\n" );
+        return -2;
+    }
+
+    result = usb_control_msg( device->handle,
           /* bmRequestType */ USB_ENDPOINT_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
           /* bRequest      */ DFU_UPLOAD,
           /* wValue        */ transaction++,
-          /* wIndex        */ interface,
-          /* Data          */ data,
+          /* wIndex        */ device->interface,
+          /* Data          */ (char *) data,
           /* wLength       */ length,
                               DFU_TIMEOUT );
 
@@ -176,18 +200,22 @@ int dfu_upload( struct usb_dev_handle *handle,
 /*
  *  DFU_GETSTATUS Request (DFU Spec 1.0, Section 6.1.2)
  *
- *  handle    - the usb_dev_handle to communicate with
- *  interface - the interface to communicate with
+ *  device    - the dfu device to commmunicate with
  *  status    - the data structure to be populated with the results
  *
  *  return the 0 if successful or < 0 on an error
  */
-int dfu_get_status( struct usb_dev_handle *handle,
-                    const unsigned short interface,
-                    struct dfu_status *status )
+int32_t dfu_get_status( dfu_device_t *device, dfu_status_t *status )
 {
     char buffer[6];
-    int result;
+    int32_t result;
+
+    TRACE( "%s( %p, %p )\n", __FUNCTION__, device, status );
+
+    if( (NULL == device) || (NULL == device->handle) ) {
+        DEBUG( "Invalid parameter\n" );
+        return -1;
+    }
 
     /* Initialize the status data structure */
     status->bStatus       = DFU_STATUS_ERROR_UNKNOWN;
@@ -195,11 +223,11 @@ int dfu_get_status( struct usb_dev_handle *handle,
     status->bState        = STATE_DFU_ERROR;
     status->iString       = 0;
 
-    result = usb_control_msg( handle,
+    result = usb_control_msg( device->handle,
           /* bmRequestType */ USB_ENDPOINT_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
           /* bRequest      */ DFU_GETSTATUS,
           /* wValue        */ 0,
-          /* wIndex        */ interface,
+          /* wIndex        */ device->interface,
           /* Data          */ buffer,
           /* wLength       */ 6,
                               DFU_TIMEOUT );
@@ -227,7 +255,7 @@ int dfu_get_status( struct usb_dev_handle *handle,
         if( 0 < result ) {
             /* There was an error, we didn't get the entire message. */
             DEBUG( "result: %d\n", result );
-            return -1;
+            return -2;
         }
     }
 
@@ -238,21 +266,26 @@ int dfu_get_status( struct usb_dev_handle *handle,
 /*
  *  DFU_CLRSTATUS Request (DFU Spec 1.0, Section 6.1.3)
  *
- *  handle    - the usb_dev_handle to communicate with
- *  interface - the interface to communicate with
+ *  device    - the dfu device to commmunicate with
  *
  *  return 0 or < 0 on an error
  */
-int dfu_clear_status( struct usb_dev_handle *handle,
-                      const unsigned short interface )
+int32_t dfu_clear_status( dfu_device_t *device )
 {
-    int result;
+    int32_t result;
 
-    result = usb_control_msg( handle,
+    TRACE( "%s( %p )\n", __FUNCTION__, device );
+
+    if( (NULL == device) || (NULL == device->handle) ) {
+        DEBUG( "Invalid parameter\n" );
+        return -1;
+    }
+
+    result = usb_control_msg( device->handle,
           /* bmRequestType */ USB_ENDPOINT_OUT| USB_TYPE_CLASS | USB_RECIP_INTERFACE,
           /* bRequest      */ DFU_CLRSTATUS,
           /* wValue        */ 0,
-          /* wIndex        */ interface,
+          /* wIndex        */ device->interface,
           /* Data          */ NULL,
           /* wLength       */ 0,
                               DFU_TIMEOUT );
@@ -266,25 +299,27 @@ int dfu_clear_status( struct usb_dev_handle *handle,
 /*
  *  DFU_GETSTATE Request (DFU Spec 1.0, Section 6.1.5)
  *
- *  handle    - the usb_dev_handle to communicate with
- *  interface - the interface to communicate with
- *  length    - the maximum number of bytes to receive from the USB
- *              device - must be less than wTransferSize
- *  data      - the buffer to put the received data in
+ *  device    - the dfu device to commmunicate with
  *
  *  returns the state or < 0 on error
  */
-int dfu_get_state( struct usb_dev_handle *handle,
-                   const unsigned short interface )
+int32_t dfu_get_state( dfu_device_t *device )
 {
-    int result;
+    int32_t result;
     char buffer[1];
 
-    result = usb_control_msg( handle,
+    TRACE( "%s( %p )\n", __FUNCTION__, device );
+
+    if( (NULL == device) || (NULL == device->handle) ) {
+        DEBUG( "Invalid parameter\n" );
+        return -1;
+    }
+
+    result = usb_control_msg( device->handle,
           /* bmRequestType */ USB_ENDPOINT_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
           /* bRequest      */ DFU_GETSTATE,
           /* wValue        */ 0,
-          /* wIndex        */ interface,
+          /* wIndex        */ device->interface,
           /* Data          */ buffer,
           /* wLength       */ 1,
                               DFU_TIMEOUT );
@@ -304,21 +339,26 @@ int dfu_get_state( struct usb_dev_handle *handle,
 /*
  *  DFU_ABORT Request (DFU Spec 1.0, Section 6.1.4)
  *
- *  handle    - the usb_dev_handle to communicate with
- *  interface - the interface to communicate with
+ *  device    - the dfu device to commmunicate with
  *
  *  returns 0 or < 0 on an error
  */
-int dfu_abort( struct usb_dev_handle *handle,
-               const unsigned short interface )
+int32_t dfu_abort( dfu_device_t *device )
 {
-    int result;
+    int32_t result;
 
-    result = usb_control_msg( handle,
+    TRACE( "%s( %p )\n", __FUNCTION__, device );
+
+    if( (NULL == device) || (NULL == device->handle) ) {
+        DEBUG( "Invalid parameter\n" );
+        return -1;
+    }
+
+    result = usb_control_msg( device->handle,
           /* bmRequestType */ USB_ENDPOINT_OUT | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
           /* bRequest      */ DFU_ABORT,
           /* wValue        */ 0,
-          /* wIndex        */ interface,
+          /* wIndex        */ device->interface,
           /* Data          */ NULL,
           /* wLength       */ 0,
                               DFU_TIMEOUT );
@@ -335,21 +375,23 @@ int dfu_abort( struct usb_dev_handle *handle,
  *
  *  vendor  - the vender number of the device to look for
  *  product - the product number of the device to look for
- *  [out] handle - the usb_dev_handle to communicate with
- *  [out] interface - the interface to communicate with
+ *  [out] device - the dfu device to commmunicate with
  *
  *  return a pointer to the usb_device if found, or NULL otherwise
  */
-struct usb_device *dfu_device_init( const unsigned int vendor,
-                                    const unsigned int product,
-                                    struct usb_dev_handle **handle,
-                                    unsigned short *interface,
-                                    const bool initial_abort,
-                                    const bool honor_interfaceclass )
+struct usb_device *dfu_device_init( const uint32_t vendor,
+                                    const uint32_t product,
+                                    dfu_device_t *dfu_device,
+                                    const dfu_bool initial_abort,
+                                    const dfu_bool honor_interfaceclass )
 {
     struct usb_bus *usb_bus;
     struct usb_device *device;
-    int retries = 4;
+    int32_t retries = 4;
+
+    TRACE( "%s( %u, %u, %p, %s, %s )\n", __FUNCTION__, vendor, product,
+           dfu_device, ((true == initial_abort) ? "true" : "false"),
+           ((true == honor_interfaceclass) ? "true" : "false") );
 
 retry:
 
@@ -363,19 +405,18 @@ retry:
                 if(    (vendor  == device->descriptor.idVendor)
                     && (product == device->descriptor.idProduct) )
                 {
-                    int tmp;
+                    int32_t tmp;
                     /* We found a device that looks like it matches...
                      * let's try to find the DFU interface, open the device
                      * and claim it. */
                     tmp = dfu_find_interface( device, honor_interfaceclass );
                     if( 0 <= tmp ) {
                         /* The interface is valid. */
-                        *interface = (unsigned short) tmp;
-                        *handle = usb_open( device );
-                        if( NULL != *handle ) {
-                            if( 0 == usb_claim_interface(*handle, *interface) ) {
-                                switch( dfu_make_idle(*handle, *interface,
-                                                      initial_abort) )
+                        dfu_device->interface = tmp;
+                        dfu_device->handle = usb_open( device );
+                        if( NULL != dfu_device->handle ) {
+                            if( 0 == usb_claim_interface(dfu_device->handle, dfu_device->interface) ) {
+                                switch( dfu_make_idle(dfu_device, initial_abort) )
                                 {
                                     case 0:
                                         return device;
@@ -385,12 +426,12 @@ retry:
                                 }
 
                                 DEBUG( "Failed to put the device in dfuIDLE mode.\n" );
-                                usb_release_interface( *handle, *interface );
-                                usb_close( *handle );
+                                usb_release_interface( dfu_device->handle, dfu_device->interface );
+                                usb_close( dfu_device->handle );
                                 retries = 4;
                             } else {
                                 DEBUG( "Failed to claim the DFU interface.\n" );
-                                usb_close( *handle );
+                                usb_close( dfu_device->handle );
                             }
                         } else {
                             DEBUG( "Failed to open device.\n" );
@@ -403,8 +444,8 @@ retry:
         }
     }
 
-    *handle = NULL;
-    *interface = 0;
+    dfu_device->handle = NULL;
+    dfu_device->interface = 0;
 
     return NULL;
 }
@@ -417,7 +458,7 @@ retry:
  *
  *  returns the state name or "unknown state"
  */
-char* dfu_state_to_string( const int state )
+char* dfu_state_to_string( const int32_t state )
 {
     char *message = "unknown state";
 
@@ -468,7 +509,7 @@ char* dfu_state_to_string( const int state )
  *
  *  returns the status name or "unknown status"
  */
-char* dfu_status_to_string( const int status )
+char* dfu_status_to_string( const int32_t status )
 {
     char *message = "unknown status";
 
@@ -537,10 +578,10 @@ char* dfu_status_to_string( const int status )
  *
  *  returns the interface number if found, < 0 otherwise
  */
-static int dfu_find_interface( const struct usb_device *device,
-                               const bool honor_interfaceclass )
+static int32_t dfu_find_interface( const struct usb_device *device,
+                                   const dfu_bool honor_interfaceclass )
 {
-    int c, i;
+    int32_t c, i;
     struct usb_config_descriptor *config;
     struct usb_interface_descriptor *interface;
 
@@ -576,25 +617,23 @@ static int dfu_find_interface( const struct usb_device *device,
 /*
  *  Gets the device into the dfuIDLE state if possible.
  *
- *  handle - the usb_dev_handle to communicate with
- *  interface - the interface to communicate with
+ *  device    - the dfu device to commmunicate with
  *
  *  returns 0 on success, 1 if device was reset, error otherwise
  */
-static int dfu_make_idle( struct usb_dev_handle *handle,
-                          unsigned short interface,
-                          bool initial_abort )
+static int32_t dfu_make_idle( dfu_device_t *device,
+                              const dfu_bool initial_abort )
 {
-    struct dfu_status status;
-    int retries = 4;
+    dfu_status_t status;
+    int32_t retries = 4;
 
     if( true == initial_abort ) {
-        dfu_abort( handle, interface );
+        dfu_abort( device );
     }
 
     while( 0 < retries ) {
-        if( 0 != dfu_get_status(handle, interface, &status) ) {
-            dfu_clear_status( handle, interface );
+        if( 0 != dfu_get_status(device, &status) ) {
+            dfu_clear_status( device );
             continue;
         }
 
@@ -607,7 +646,7 @@ static int dfu_make_idle( struct usb_dev_handle *handle,
                 }
 
                 /* We need the device to have the DFU_STATUS_OK status. */
-                dfu_clear_status( handle, interface );
+                dfu_clear_status( device );
                 break;
 
             case STATE_DFU_DOWNLOAD_SYNC:   /* abort -> idle */
@@ -616,21 +655,21 @@ static int dfu_make_idle( struct usb_dev_handle *handle,
             case STATE_DFU_UPLOAD_IDLE:     /* abort -> idle */
             case STATE_DFU_DOWNLOAD_BUSY:   /* abort -> error */
             case STATE_DFU_MANIFEST:        /* abort -> error */
-                dfu_abort( handle, interface );
+                dfu_abort( device );
                 break;
 
             case STATE_DFU_ERROR:
-                dfu_clear_status( handle, interface );
+                dfu_clear_status( device );
                 break;
 
             case STATE_APP_IDLE:
-                dfu_detach( handle, interface, DFU_DETACH_TIMEOUT );
+                dfu_detach( device, DFU_DETACH_TIMEOUT );
                 break;
 
             case STATE_APP_DETACH:
             case STATE_DFU_MANIFEST_WAIT_RESET:
                 DEBUG( "Resetting the device\n" );
-                usb_reset( handle );
+                usb_reset( device->handle );
                 return 1;
         }
 
@@ -649,7 +688,7 @@ static int dfu_make_idle( struct usb_dev_handle *handle,
  *  function - the calling function to output on behalf of
  *  result   - the result to interpret
  */
-static void dfu_msg_response_output( const char *function, const int result )
+static void dfu_msg_response_output( const char *function, const int32_t result )
 {
     char *msg = NULL;
 
