@@ -1,7 +1,7 @@
 /*
  * dfu-programmer
  *
- * $Id: dfu.h 48 2007-03-28 07:35:45Z schmidtw $
+ * $Id: dfu.h 71 2008-10-02 05:48:41Z schmidtw $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,11 @@
 #define __DFU_H__
 
 #include <usb.h>
-#include <stdbool.h>
+#include <stdint.h>
+#include <stddef.h>
+
+#include "dfu-bool.h"
+#include "dfu-device.h"
 
 /* DFU states */
 #define STATE_APP_IDLE                  0x00
@@ -65,40 +69,27 @@
  *  1 unsigned byte iString
 */
 
-struct dfu_status {
-    unsigned char bStatus;
-    unsigned int  bwPollTimeout;
-    unsigned char bState;
-    unsigned char iString;
-};
+typedef struct {
+    uint8_t bStatus;
+    uint32_t bwPollTimeout;
+    uint8_t bState;
+    uint8_t iString;
+} dfu_status_t;
 
-int dfu_detach( struct usb_dev_handle *handle,
-                const unsigned short interface,
-                const unsigned short timeout );
-int dfu_download( struct usb_dev_handle *handle,
-                  const unsigned short interface,
-                  const unsigned short length,
-                  char* data );
-int dfu_upload( struct usb_dev_handle *handle,
-                const unsigned short interface,
-                const unsigned short length,
-                char* data );
-int dfu_get_status( struct usb_dev_handle *handle,
-                    const unsigned short interface,
-                    struct dfu_status *status );
-int dfu_clear_status( struct usb_dev_handle *handle,
-                      const unsigned short interface );
-int dfu_get_state( struct usb_dev_handle *handle,
-                   const unsigned short interface );
-int dfu_abort( struct usb_dev_handle *handle,
-               const unsigned short interface );
-struct usb_device *dfu_device_init( const unsigned int vendor,
-                                    const unsigned int product,
-                                    struct usb_dev_handle **handle,
-                                    unsigned short *interface,
-                                    const bool initial_abort,
-                                    const bool honor_interfaceclass );
+int32_t dfu_detach( dfu_device_t *device, const int32_t timeout );
+int32_t dfu_download( dfu_device_t *device, const size_t length, uint8_t* data );
+int32_t dfu_upload( dfu_device_t *device, const size_t length, uint8_t* data );
+int32_t dfu_get_status( dfu_device_t *device, dfu_status_t *status );
+int32_t dfu_clear_status( dfu_device_t *device );
+int32_t dfu_get_state( dfu_device_t *device );
+int32_t dfu_abort( dfu_device_t *device );
 
-char* dfu_status_to_string( const int status );
-char* dfu_state_to_string( const int state );
+struct usb_device *dfu_device_init( const uint32_t vendor,
+                                    const uint32_t product,
+                                    dfu_device_t *device,
+                                    const dfu_bool initial_abort,
+                                    const dfu_bool honor_interfaceclass );
+
+char* dfu_status_to_string( const int32_t status );
+char* dfu_state_to_string( const int32_t state );
 #endif
